#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""Fix lingering params issues

Revision ID: 258d71a32328
Revises: 41a10866241c
Create Date: 2022-02-04 09:49:12.019437

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "258d71a32328"
down_revision = "41a10866241c"
branch_labels = None
depends_on = None

script = b"""#!/usr/bin/env bash

set -o errexit -o nounset

# This script is for resubmitting a manually altered standard calibration.
# A new capability request version will be created and all relevant files
# from the previous version results (contained in the current directory) will be appended to it.

CAPABILITY_SERVICE=$(capo -q edu.nrao.workspaces.CapabilitySettings.externalServiceUrl)

# create new request version
data_location={{spool_dir}}/rawdata
if [[ -L $data_location ]]; then
    qa="qa2"
    corrected_location="${data_location/spool/"$qa"}"
    params="{\\"resubmit\\": true, \\"rawdata_dir\\":\\"location\\"}"
    corrected_params="${params/location/"$corrected_location"}"
    capability_request_id=$(curl --header "Content-Type: application/json" --request POST --data "$corrected_params" $CAPABILITY_SERVICE/capability/request/execution/{{request_id}}/create_version)
else
    params="{\\"resubmit\\": true, \\"rawdata_dir\\":\\"location\\"}"
    corrected_params="${params/location/"$data_location"}"
    capability_request_id=$(curl --header "Content-Type: application/json" --request POST --data "$corrected_params" $CAPABILITY_SERVICE/capability/request/execution/{{request_id}}/create_version)
fi
# add all files in current directory to new version

# sort files for valid additions
echo "Sorting files...."
mkdir "system_logs"
mv *.{err,out,log,dag,condor} ./system_logs
mv std_calibration* ./system_logs
mv _condor* ./system_logs
echo "Files sorted. Continuing...."

for appendFile in $PWD/*; do
  if [[ -f "${appendFile}" ]] && [[ "${appendFile##*/}" != "resubmit.sh" ]] && [[ "${appendFile##*/}" != "metadata.json" ]]; then
    echo "Found a file to add..."
    filename="${appendFile##*/}"
    curl -X PUT --data-binary "@$appendFile" $CAPABILITY_SERVICE/capability/request/$capability_request_id/version/latest/files/$filename
  fi
done
echo "All files added."

# submit new version for execution
curl -X POST $CAPABILITY_SERVICE/capability/request/$capability_request_id/submit

"""

old_script = b"""#!/usr/bin/env bash

set -o errexit -o nounset

# This script is for resubmitting a manually altered standard calibration.
# A new capability request version will be created and all relevant files
# from the previous version results (contained in the current directory) will be appended to it.

CAPABILITY_SERVICE=$(capo -q edu.nrao.workspaces.CapabilitySettings.serviceUrl)

# create new request version
data_location={{spool_dir}}/rawdata
if [[ -L $data_location ]]; then
    qa="qa2"
    corrected_location="${data_location/spool/"$qa"}"
    params="{\\"resubmit\\": true, \\"rawdata_dir\\":\\"location\\"}"
    corrected_params="${params/location/"$corrected_location"}"
    capability_request_id=$(curl --header "Content-Type: application/json" --request POST --data "$corrected_params" $CAPABILITY_SERVICE/capability/request/execution/{{request_id}}/create_version)
else
    params="{\\"resubmit\\": true, \\"rawdata_dir\\":\\"location\\"}"
    corrected_params="${params/location/"$data_location"}"
    capability_request_id=$(curl --header "Content-Type: application/json" --request POST --data "$corrected_params" $CAPABILITY_SERVICE/capability/request/execution/{{request_id}}/create_version)
fi
# add all files in current directory to new version

# sort files for valid additions
echo "Sorting files...."
mkdir "system_logs"
mv *.{err,out,log,dag,condor} ./system_logs
mv std_calibration* ./system_logs
echo "Files sorted. Continuing...."

for appendFile in $PWD/*; do
  if [[ -f "${appendFile}" ]] && [[ "${appendFile##*/}" != "resubmit.sh" ]] && [[ "${appendFile##*/}" != "metadata.json" ]]; then
    echo "Found a file to add..."
    filename="${appendFile##*/}"
    curl -X PUT --data-binary "@$appendFile" $CAPABILITY_SERVICE/capability/request/$capability_request_id/version/latest/files/$filename
  fi
done
echo "All files added."

# submit new version for execution
curl -X POST $CAPABILITY_SERVICE/capability/request/$capability_request_id/submit

"""

carta_debug = b"""
#!/bin/sh
set -o errexit

export HOME=$_CONDOR_SCRATCH_DIR
{{#remote-cv}}
export CAPO_PATH=$(readlink -f $_CONDOR_SCRATCH_DIR)
export CAPO_PROFILE={{profile}}
{{/remote-cv}}
fetch_foundation(){
  ../productfetcher{{product_locator}}

  echo "Moving images out of subdirectories...."
  for dir in *.fits; do mv $dir $dir.1; done;

  find . -mindepth 1 -type f -print -exec mv {} . \\;
  find . -mindepth 1 -type d -empty -delete
  echo "Done. Ready for Carta!"
}


{{#need_data}}
./framework.sh -r .
chmod 770 .
cd rawdata/

fetch_foundation {{^single_image}}&{{/single_image}}

.{{/need_data}}./carta_envoy {{#need_data}}--parallel{{/need_data}} {{#single_image}}--single -i {{imageName}}{{/single_image}} -d $1 -n $2 -wf $3 -r $4
{{#need_data}}wait{{/need_data}}
"""


def upgrade():
    op.get_bind().execute(
        f"""
        UPDATE workflow_templates
        SET content=%s WHERE filename='resubmit.sh' AND workflow_name='std_calibration'
        """,
        script,
    )
    op.get_bind().execute(
        f"""
        UPDATE workflow_templates
        SET content=%s WHERE filename='carta.sh' AND workflow_name='carta-debug'
        """,
        carta_debug,
    )


def downgrade():
    op.get_bind().execute(
        f"""
        UPDATE workflow_templates
        SET content=%s WHERE filename='resubmit.sh' AND workflow_name='std_calibration'
        """,
        old_script,
    )
